@@ -8,6 +8,7 @@ import re
 
 import easychart.encoders as encoders
 import easychart.internals as internals
+import easychart.config
 
 
 class SeriesCollection(easytree.Tree):
@@ -70,10 +71,14 @@ class SeriesCollection(easytree.Tree):
                 kwargs["name"] = data.name
 
         if isinstance(data, (pd.Series, pd.DataFrame)):
-            if "index" in kwargs and isinstance(
-                kwargs["index"], collections.abc.Iterable
-            ):
-                data = data.values.tolist()
+            if "index" in kwargs:
+                if isinstance(kwargs["index"], collections.abc.Iterable):
+                    data = data.values.tolist()
+                elif isinstance(kwargs["index"], bool):
+                    if kwargs["index"] == False:
+                        data = data.values.tolist()
+                    if kwargs["index"] == True:
+                        data = data.reset_index().values.tolist()
             else:
                 data = data.reset_index().values.tolist()
 
@@ -508,7 +513,211 @@ class Chart(easytree.Tree):
 
         return self.plot(reg.predict(X), index=x, **kwargs)
 
-    def show(self, width="100%", height="400px", theme=None):
+    def annotate(
+        self,
+        text,
+        *,
+        x=0,
+        y=0,
+        xOffset=None,
+        yOffset=None,
+        point=None,
+        align=None,
+        verticalAlign=None,
+        shape=None,
+        padding=None,
+        distance=None,
+        backgroundColor=None,
+        borderColor=None,
+        borderRadius=None,
+        borderWidth=None,
+        useHTML=None,
+        size=None,
+        color=None,
+        xAxis=0,
+        yAxis=0,
+        width=None,
+        draggable="xy",
+        zIndex=None,
+        visible=None,
+    ):
+        """
+        Add an annotation to a chart
+
+        Parameters
+        ----------
+        text : str
+            The annotation label text
+        x : float
+            The x position of the point to which the annotation refers to 
+        y : float
+            The y position of the point to which the annotation refers to
+        xAxis : int
+            The x-Axis in which the x coordinate is defined
+        yAxis : int
+            The y-Axis in which the y coordinate is defined
+        xOffset : float
+            The horizontal offset of the annotation relative to the point (in pixels)
+        yOffset : float
+            The vertical offset of the annotation relative to the point (in pixels)
+        width : int
+            The width (in pixels) of the annotation
+        point : str
+            The id of the point to which the annotation refers to (if x and y not specified)
+        align : str (one of "center", "left", "right")
+            The horizontal alignment of the annotation relative to the point
+        verticalAlign: str (one of "bottom", "middle", "top")
+            The vertical alignment of the annotation relative to the point
+        shape : str (one of "callout", "connector", "rect", "circle", "diamond", "triangle")
+            The shape of the annotation
+        padding : int
+            The padding within the box (assuming border or background is set)
+        distance : float
+            The offset of the annotation relative to the point (in lieu of xOffset and yOffset)
+        size : int
+            The font size
+        color : str
+            The font color
+        
+        Note
+        ----
+        For more details on annotations, check out the `Highcharts API documentation <https://api.highcharts.com/highcharts/annotations.labelOptions>`_
+        """
+        if point is None:
+            point = {"x": x, "y": y, "xAxis": xAxis, "yAxis": yAxis}
+        elif isinstance(point, tuple):
+            point = dict(zip(["x", "y", "xAxis", "yAxis"], [*point, 0, 0, 0, 0]))
+        elif isinstance(point, dict):
+            point = {"xAxis": xAxis, "yAxis": yAxis, **point}
+
+        with self.annotations.append({}) as annotation:
+            annotation.labels.append(
+                internals.dictfilter(
+                    {
+                        "text": text,
+                        "point": point,
+                        "x": xOffset,
+                        "y": -yOffset if yOffset is not None else yOffset,
+                        "align": align,
+                        "verticalAlign": verticalAlign,
+                        "shape": shape,
+                        "padding": padding,
+                        "backgroundColor": backgroundColor,
+                        "borderColor": borderColor,
+                        "borderRadius": borderRadius,
+                        "borderWidth": borderWidth,
+                        "useHTML": useHTML,
+                        "distance": distance,
+                        "style": internals.dictfilter(
+                            {"fontSize": size, "color": color, "width": width},
+                            lambda k, v: v is not None,
+                        ),
+                    },
+                    lambda k, v: v is not None,
+                )
+            )
+
+            annotation.labelOptions = internals.dictfilter(
+                {"visible": visible, "draggable": draggable, "zIndex": zIndex},
+                lambda k, v: v is not None,
+            )
+
+    def draw(
+        self,
+        shape,
+        *,
+        x=None,
+        y=None,
+        xAxis=0,
+        yAxis=0,
+        r=None,
+        width=None,
+        height=None,
+        xOffset=None,
+        yOffset=None,
+        point=None,
+        points=None,
+        draggable="xy",
+        visible=None,
+        zIndex=None,
+        **kwargs,
+    ):
+        # easychart feature: drawing line segments
+        if shape == "line":
+            shape = "path"
+
+            if "fill" not in kwargs:
+                kwargs["fill"] = False
+
+            if "color" in kwargs:
+                kwargs["stroke"] = kwargs.pop("color")
+
+            if "width":
+                kwargs["strokeWidth"], width = width, None
+
+            if "linestyle" in kwargs:
+                kwargs["dashStyle"] = kwargs.pop("linestyle")
+
+            if "style" in kwargs:
+                kwargs["dashStyle"] = kwargs.pop("style")
+
+        # easychart feature: drawing boxes (rectangles defined in the x/y space)
+        if shape == "box":
+            shape = "path"
+            points = [
+                (x, y),
+                (x + width, y),
+                (x + width, y + height),
+                (x, y + height),
+                (x, y),
+            ]
+
+        if shape == "square" or shape == "rectangle":
+            shape = "rect"
+
+        if "color" in kwargs:
+            kwargs["fill"] = kwargs.pop("color")
+
+        # inject default axes by default
+        if points is not None:
+            for i, p in enumerate(points):
+                if isinstance(p, (tuple)):
+                    points[i] = dict(
+                        zip(["x", "y", "xAxis", "yAxis"], [*p, 0, 0, 0, 0])
+                    )
+
+                elif isinstance(p, dict):
+                    points[i] = {"yAxis": yAxis, "xAxis": xAxis, **p}
+
+        with self.annotations.append({}) as annotation:
+            annotation.shapes.append(
+                internals.dictfilter(
+                    {
+                        "type": shape,
+                        "point": point
+                        or (
+                            None
+                            if (x == y == None)
+                            else {"x": x, "y": y, "xAxis": xAxis, "yAxis": yAxis}
+                        ),
+                        "x": xOffset,
+                        "y": -yOffset if yOffset is not None else yOffset,
+                        "points": points,
+                        "r": r,
+                        "height": height,
+                        "width": width,
+                        **kwargs,
+                    },
+                    lambda k, v: v is not None,
+                )
+            )
+            annotation.shapeOptions = internals.dictfilter(
+                {"draggable": draggable, "zIndex": zIndex, "visible": visible},
+                lambda k, v: v is not None,
+            )
+        pass
+
+    def show(self, width=None, height=None, theme=None):
         return Plot(self, width, height, theme)
 
     def save(self, filename, indent=4):
@@ -537,7 +746,7 @@ class Plot:
     Chart container
     """
 
-    def __init__(self, chart, width="100%", height="400px", theme=None):
+    def __init__(self, chart, width=None, height=None, theme=None):
         """
         Parameters
         ------------
@@ -549,27 +758,37 @@ class Plot:
             dictionary of global options (theme)
         """
         self.chart = chart
-        if not isinstance(width, str) and not width.endswith("%"):
-            width = f"{width}%"
-        self.width = width
-        if not isinstance(height, str) and not height.endswith("px"):
-            height = f"{height}px"
-        self.height = height
+
+        if height is None:
+            if "chart" in chart and "height" in chart["chart"]:
+                self.height = internals.Size(chart["chart"]["height"])
+            else:
+                self.height = internals.Size("400px")
+        else:
+            self.height = internals.Size(height)
+
+        if width is None:
+            if "chart" in chart and "width" in chart["chart"]:
+                self.width = internals.Size(chart["chart"].pop("width"))
+            else:
+                self.width = internals.Size("100%")
+        else:
+            if "chart" in chart and "width" in chart["chart"]:
+                chart["chart"].pop("width")
+
+            self.width = internals.Size(width)
+
         self.theme = theme
 
     def __repr__(self):
         return f"<Plot height={self.height} width={self.width}>"
 
     def serialize(self):
-        if isinstance(self.chart, dict):
-            return {"chart": self.chart, "width": self.width, "height": self.height}
-        if isinstance(self.chart, (Chart, easytree.Tree)):
-            return {
-                "chart": self.chart.serialize(),
-                "width": self.width,
-                "height": self.height,
-            }
-        raise NotImplementedError
+        return {
+            "chart": self.chart,
+            "width": self.width,
+            "height": self.height,
+        }
 
 
 class Grid:
@@ -577,7 +796,7 @@ class Grid:
     Grid of chart plots
     """
 
-    def __init__(self, plots=None, theme=None):
+    def __init__(self, plots=None, theme=None, width="100%", responsive=None):
         """
         Parameters
         ------------------------
@@ -586,10 +805,20 @@ class Grid:
         theme : dict (optional)
             dictionary of global options (theme)
         """
-        self.plots = plots or []
+        self.plots = (
+            [p if isinstance(p, Plot) else Plot(p) for p in plots]
+            if plots is not None
+            else []
+        )
         self.theme = theme
 
-    def add(self, chart, width="100%", height="400px"):
+        self.width = internals.Size(width).resolve(
+            easychart.config.config.rendering.container.width
+        )
+
+        self.responsive = responsive
+
+    def add(self, chart, width=None, height=None):
         """
         Adds a chart (or plot) to the grid
 
@@ -607,18 +836,39 @@ class Grid:
     @property
     def height(self):
         # extract the height and width of each plot
-        heights = [int(plot.height[:-2]) for plot in self.plots]
-        widths = [int(plot.width[:-1]) for plot in self.plots]
-        # group the plots by row
-        cumwidth, rows = 0, []
-        for i, (height, width) in enumerate(zip(heights, widths)):
-            if i == 0:
+        dimensions = [
+            (
+                internals.Size(plot.height).resolve(self.width),
+                internals.Size(plot.width).resolve(self.width),
+            )
+            for plot in self.plots
+        ]
+
+        cumwidth, rows = 0, [[]]
+        for (height, width) in dimensions:
+            # if there are less than 0 pixels left, put on a new row
+            if (self.width - (cumwidth + width)) < 0:
                 rows.append([height])
+                cumwidth = width
             else:
-                if (cumwidth + width) <= 100:
-                    rows[-1].append(height)
-                else:
-                    rows.append([height])
-                    cumwidth = 0
-            cumwidth += width
-        return f"{sum([max(row) for row in rows])}px"
+                rows[-1].append(height)
+                cumwidth += width
+
+        return f"{sum([max(row) if len(row) > 0 else 0 for row in rows])}px"
+
+    def serialize(self):
+        responsive = (
+            easychart.config.config.rendering.responsive
+            if self.responsive is None
+            else self.responsive
+        )
+
+        plots = [plot.serialize() for plot in self.plots]
+
+        for plot in plots:
+            if not responsive:
+                for dimension in ("width", "height"):
+                    plot[dimension] = f"{plot[dimension].resolve(self.width)}px"
+
+        return plots
+
